/// <reference types="node" />
import { EventEmitter } from 'events';
import { type Model, type RootModel } from 'racer';
import { util } from 'racer';
import { type ComponentConstructor, type SingletonComponentConstructor } from './components';
import { type Derby } from './Derby';
import { PageForClient, type Page } from './Page';
import { PageParams } from './routes';
import { type Views } from './templates/templates';
declare module 'racer/lib/util' {
    let isProduction: boolean;
}
export declare function createAppPage(derby: any): typeof Page;
export interface AppOptions {
    appMetadata?: Record<string, string>;
    scriptHash?: string;
}
type OnRouteCallback = (arg0: PageForClient, arg1: PageForClient, model: Model, params: PageParams, done?: () => void) => void;
type Routes = [string, string, any][];
/**
 * APP EVENTS
 *
  'error', Error
  'pageRendered', Page
  'destroy'
  'model', Model
  'route', Page
  'routeDone', Page, transition: boolean
  'ready', Page
  'load', Page
  'destroyPage', Page
 */
export declare abstract class AppBase extends EventEmitter {
    derby: Derby;
    name: string;
    filename: string;
    scriptHash: string;
    appMetadata: Record<string, string>;
    Page: typeof Page;
    proto: any;
    views: Views;
    tracksRoutes: Routes;
    model: RootModel;
    page: Page;
    protected _pendingComponentMap: Record<string, ComponentConstructor | SingletonComponentConstructor>;
    protected _waitForAttach: boolean;
    protected _cancelAttach: boolean;
    use: typeof util.use;
    serverUse: typeof util.serverUse;
    constructor(derby: any, name?: string, filename?: string, options?: AppOptions);
    abstract _init(options?: AppOptions): any;
    loadViews(_viewFilename: any, _viewName?: any): void;
    loadStyles(_filename: any, _options?: any): void;
    component(constructor: ComponentConstructor | SingletonComponentConstructor): this;
    component(name: string, constructor: ComponentConstructor | SingletonComponentConstructor, isDependency?: boolean): this;
    addViews(_viewFileName: string, _namespace: string): void;
    onRoute(callback: OnRouteCallback, page: PageForClient, next: () => void, done: () => void): void;
}
export declare class App extends AppBase {
    page: PageForClient;
    history: {
        refresh(): void;
        push(url: string): void;
        replace(url: string): void;
    };
    constructor(derby: any, name: any, filename: any, options: AppOptions);
    _init(_options: any): void;
    private _views;
    private _finishInit;
    private _getAppData;
    private _contentReady;
    private _getAppStateScript;
    createPage(): PageForClient;
    private _destroyCurrentPage;
    private _autoRefresh;
    private _handleMessage;
    static _parseInitialData(jsonString: string): any;
}
export {};
